package dev.fyloz.trial.colorrecipesexplorer.web.controller;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.JSONResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.RecipeExplorerFormDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.*;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.*;

@Controller
public class RecipeExplorerController {

    private RecipeService recipeService;
    private MixService mixService;

    @Autowired
    public RecipeExplorerController(RecipeService recipeService, MixService mixService) {
        this.recipeService = recipeService;
        this.mixService = mixService;
    }

    /**
     * Affiche la recette.
     * Cette méthode requiert l'identifiant de la recette à affiche dans l'URL.
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipe: Contient la recette
     * - mixes: Contient les mélanges associées à la recette
     * - images: Contient la liste des noms des images associées à la recette
     *
     * @param id L'identifiant de la recette
     * @return La page à afficher.
     */
    @GetMapping(EXPLORER_RECIPE_SPECIFIC)
    public ModelAndView getPage(@PathVariable Long id) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(EXPLORER_RECIPE);

        try {
            Recipe recipe = recipeService.getById(id);

            modelResponseBuilder.addResponseData(ResponseDataType.RECIPE, recipe);

            List<Mix> mixes = new ArrayList<>(recipe.getMixes()); // Convertit le PersistentBag en ArrayList
            mixes.sort(Comparator.comparing(Mix::getId));

            return modelResponseBuilder
                    .addResponseData(ResponseDataType.RECIPE, recipe)
                    .addResponseData(ResponseDataType.MIXES, mixes)
                    .addResponseData(ResponseDataType.IMAGES, recipeService.getImageFiles(recipe))
                    .build();
        } catch (EntityNotFoundException e) {
            return modelResponseBuilder
                    .withView(INDEX)
                    .addResponseCode(ResponseCode.RECIPE_NOT_FOUND, id)
                    .build();
        }
    }

    @PostMapping(value = EXPLORER_RECIPE, consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Map<String, Object> saveRecipeInformations(RecipeExplorerFormDto form) {
        JSONResponseBuilder responseBuilder = new JSONResponseBuilder();

        try {

            responseBuilder.addResponseCode(ResponseCode.SUCCESS_SAVING_RECIPE_INFORMATIONS);
        } catch (EntityNotFoundException e) {
            ResponseCode responseCode = null;
            switch (e.getType().getSimpleName()) {
                case "Recipe":
                    responseCode = ResponseCode.RECIPE_NOT_FOUND;
                    break;
                case "Mix":
                    responseCode = ResponseCode.MIX_NOT_FOUND;
                    break;
            }

            responseBuilder.addResponseCode(responseCode, e.getRequestedId());
        }

        return responseBuilder.build();
    }
}
